@@ -9,11 +9,10 @@
 #' provided location. Ancestors with frequency = 0 are dropped out of the table.
 #' The tibble contains three columns: location, ancestor and frequency.
 #' @examples
-#' number_founders = 20
-#' wildpop =  simulate_admixture(pop_size = 1000,
-#'                               number_of_founders = number_founders,
-#'                               total_runtime = 10,
-#'                               morgan = 1)
+#' wildpop =  simulate_admixture(
+#'    module = ancestry_module(number_of_founders = 20, morgan = 1),
+#'    pop_size = 1000,
+#'    total_runtime = 10)
 #'
 #' avg_frequencies <- calculate_marker_frequency(pop = wildpop,
 #'                                               location = 0.5)
@@ -51,5 +50,14 @@ calculate_marker_frequency <- function(pop, location) {
   }
   output <- output[, c("location", "ancestor", "frequency")]
 
+  using_sequencing_data <- check_for_bases(pop)
+  if (using_sequencing_data) {
+    output$ancestor[output$ancestor == 0] <- 0
+    output$ancestor[output$ancestor == 1] <- "a"
+    output$ancestor[output$ancestor == 2] <- "c"
+    output$ancestor[output$ancestor == 3] <- "t"
+    output$ancestor[output$ancestor == 4] <- "g"
+  }
+
   return(output)
 }
