@@ -11,7 +11,7 @@
 
 #include <stdio.h>
 #include <vector>
-#include <random>
+#include "random_functions.h"
 
 struct junction {
     long double pos;
@@ -41,6 +41,6 @@ struct Fish {
 };
 
 
-Fish mate(const Fish& A, const Fish& B, double numRecombinations);
+Fish mate(const Fish& A, const Fish& B, double numRecombinations, rnd_t& rndgen);
 
 #endif /* Fish_hpp */
