@@ -8,10 +8,11 @@ test_that("save_population", {
   run_time <- 10
   morgan <- 1
 
-  vx <- simulate_admixture(pop_size = pop_size,
-                             number_of_founders = number_of_founders,
-                             total_runtime = run_time,
-                             morgan = morgan)
+  vx <- simulate_admixture(module = ancestry_module(number_of_founders =
+                                                      number_of_founders,
+                                                    morgan = morgan),
+                           pop_size = pop_size,
+                           total_runtime = run_time)
 
   testthat::expect_true(verify_population(vx))
 
@@ -28,3 +29,12 @@ test_that("save_population", {
   }
   testthat::expect_true(file.remove("test.pop"))
 })
+
+test_that("data", {
+  data("dgrp2.3R.5k.data")
+  testthat::expect_equal(length(dgrp2.3R.5k.data$markers),
+                         4603)
+
+  testthat::expect_equal(length(dgrp2.3R.5k.data$genomes[, 1]),
+                         410)
+})
